#ifndef __DM_H
#define __DM_H

#include <ode/ode.h>
#include <drawstuff/drawstuff.h>

#ifdef dDOUBLE
#define dsDrawSphere   dsDrawSphereD
#define dsDrawBox      dsDrawBoxD
#define dsDrawCylinder dsDrawCylinderD
#define dsDrawCapsule  dsDrawCapsuleD
#define dsDrawLine     dsDrawLineD
#define dsDrawTriangle dsDrawTriangleD
#endif

extern void simLoop(int cmd);
extern double R[12];
extern int CN1; // ボールセンサ
extern int CN2; // 地磁気センサ

typedef struct{
    dBodyID body; // ボディのID
    dGeomID geom; // ジオメトリのID
    dJointID joint; // ジョイントのID
    const double *p; // x, y, z　[m]
    const double *R;   // 回転行列 要素数4x3
    double m; // 質量 [kg]
    double r,l; // 半径 [m], 長さ [m]
    const double *side; // サイズ　x,y,z
    const double *color; // 色 r,g,b
} dmObject;

typedef struct
{
    void (*start)();            // 初期化関数
    void (*step) (int pause);	// ステップ関数
    void (*command) (int cmd);	// キー関数
} dmFunctions;

void command(int cmd);               /*** キー入力の処理　***/
void start();                 /*** 前処理　***/
void setDrawStuff();          /*** 描画関数の設定 ***/
void setDrawStuff(dmFunctions *dmf);

void dmAddForce(dmObject *obj, double fx, double fy, double fz);
void dmAddTorque(dmObject *obj, double fx, double fy, double fz);
void dmSetCamera(double x, double y, double z, double roll, double pitch, double yaw);
void dmClose();
void dmCreateSphere(dmObject *obj, double p[3], double R[12], double m, double r, double color[3]);
void dmCreateBox(dmObject *obj, double p[3], double R[12], double m, double side[3],  double color[3]);
void dmCreateCylinder(dmObject *obj, double p[3], double R[12], double m, double r, double l, double color[3]);
void dmCreateCapsule(dmObject *obj, double p[3], double R[12], double m, double r, double l, double color[3]);
void dmInit();
void dmDrawBox(double x, double y, double z, double sx, double sy,
               double sz);
dJointID dmJointCreateFixed();
dJointID dmJointCreateHinge();
dJointID dmJointCreateSlider();
void dmLoop(int w, int h, dmFunctions *dmf);
void dmLoop(int w, int h, void (*function)(int pause), void (*function2)(int cmd));

void dmLoop();
void dmLoop(int w, int h);
void dmSimStep();
void dmSimQuickStep();
void dmDraw(dmObject *obj);

double vision(dReal *r, dReal *phi, dBodyID obj_body);
int ballSensor();



/*** 車輪の制御 ***/
void motor(int right, int left);

/***   その場回転  ***/
void turn(int power);

/*** 追跡  phi:物体の方位, speed:車輪の回転速度 ***/
void track(dReal phi, int power);

void makeRobot();
void drawBall();
void drawRobot();
void user_main();
int get_dir(int dno);
/*** wait関数　***/
void wait_ms(int ms);

#endif




